#include "udm_config.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "udm_common.h"
#include "udm_parseurl.h"
#include "udm_utils.h"

int UdmParseURL(UDM_URL *url,char *s){
char *ss;

	if(strlen(s)>=UDM_URLSIZE)
		return(2);
	*(url->schema)=0;
	*(url->specific)=0;
	*(url->hostinfo)=0;
	*(url->hostname)=0;
	*(url->anchor)=0;
	*(url->auth)=0;
	url->port=0;
	*(url->path)=0;
	*(url->filename)=0;

	if(!(ss=strchr(s,':'))){
		/* Have not scheme, relative path only */
		strcpy(url->path,s);
		strcpy(url->schema,"");
		strcpy(url->specific,"");
	}else{
		/* Have scheme - absolute path */
		*ss=0;
		strcpy(url->schema,s);
		strcpy(url->specific,ss+1);
		*ss=':';

		if(!strncmp(url->specific,"//",2)){
			char * hostname;
			/* Have hostinfo */
			if((ss=strchr(url->specific+2,'/'))){
				/* Have hostname with path */
				*ss=0;
				strcpy(url->hostinfo,url->specific+2);
				*ss='/';
				strcpy(url->path,ss);
			}else{
				/* Hostname without path */
				strcpy(url->hostname,url->specific+2);
				strcpy(url->path,"/");
			}
			if(hostname=strchr(url->hostinfo,'@')){
				/* Username and password is given  */
				/* Store auth string user:password */
				*hostname=0;
				strcpy(url->auth,url->hostinfo);
				*hostname='@';
				hostname++;
			}else{
				hostname=url->hostinfo;
			}
			if((ss=strchr(hostname,':'))){
				*ss=0;
				strcpy(url->hostname,hostname);
				*ss=':';
				url->port=atoi(ss+1);
			}else{
				strcpy(url->hostname,hostname);
				url->port=0;
			}
		}else{
			/* Have not host */
			strcpy(url->hostname,"localhost");
			strcpy(url->path,url->specific);
		}
	}

	/* Cat anchor if exist */
	if((ss=strstr(url->path,"#")))*ss=0;

	UdmRemove2Dot(url->path);
	
	/* Separate path and filename */
	if((url->path[0]!='/')&&(url->path[1]!=':')){ /* Relative path in this case */
		if(!strncmp(url->path,"./",2))
			strcpy(url->filename,url->path+2);
		else
			strcpy(url->filename,url->path);
		strcpy(url->path,"");
	}
	if((ss=strrchr(url->path,'/'))&&(strcmp(ss,"/"))){
		strcpy(url->filename,ss+1);
		*(ss+1)=0;
	}
	return(0);
}


/*********************** Tags ****************************/

int UdmFreeTag(UDM_TAG *tag){
	UDM_FREE(tag->name);
	UDM_FREE(tag->href);
	UDM_FREE(tag->src);
	UDM_FREE(tag->content);
	UDM_FREE(tag->equiv);
	UDM_FREE(tag->value);
	UDM_FREE(tag->selected);
	return(0);
}

int UdmParseTag(UDM_TAG *tag,char *stag){
char *s,*e,*p;
int len;

	len=strlen(stag);s=stag+1; e=s;
	while(!strchr(" \t\r\n>",*e))e++; 
	*e=0;
	tag->name=strdup(s);
	tag->href=NULL;
	tag->src=NULL;
	tag->content=NULL;
	tag->equiv=NULL;
	tag->value=NULL;
	tag->selected=NULL;
	e++;
	UDM_SKIP(e," \t\r\n>");
	while(e-stag<len){
		s=e;
		UDM_SKIPN(e," \t\r\n=>");
		if(*e!='='){
			*e++=0;
			UDM_SKIP(e," \t\r\n>");
		}
		if(*e=='='){
			*e=0;e++;
			UDM_SKIP(e," \t\r\n>");
			p=e;
			if(*p=='"'){
				p++;e++;
				UDM_SKIPN(e,"\">");
			}else
			if(*p=='\''){
				p++;e++;
				UDM_SKIPN(e,"'>");
			}else{
				UDM_SKIPN(e," \t\r\n>");
			}
			*e=0;e++;

			if(!strcasecmp(s,"href")){
				tag->href=strdup(p);
			}else
			if(!strcasecmp(s,"value")){
				tag->value=strdup(p);
			}else
			if(!strcasecmp(s,"selected")){
				tag->selected=strdup(p);
			}else
			if(!strcasecmp(s,"src")){
				tag->src=strdup(p);
			}else
			if(!strcasecmp(s,"content")){
				tag->content=strdup(p);
			}else
			if(!strcasecmp(s,"name")||!strcasecmp(s,"http-equiv")){
				tag->equiv=strdup(p);
			}
		}
	}return(0);
}

